@@ -2,6 +2,7 @@
 *
 * Copyright 2002 Sun Microsystems, Inc.,
 * Copyright 2002 University Of Toronto
+* Copyright 2009 Gerd Kohlberger <gerdko gmail com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
@@ -23,10 +24,10 @@
 #  include <config.h>
 #endif
 
-#include <gnome.h>
-
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
+#include <glib.h>
+#include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
 #include "gok-gconf-keys.h"
 #include "gok-log.h"
@@ -140,18 +141,36 @@ gok_input_extension_filter (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer dat
 #endif
 }
 
+static GdkWindow *
+gok_input_get_root_window (void)
+{
+  GtkWidget *main_window;
+  GdkWindow *gdk_window;
+
+  main_window = gok_main_get_main_window ();
+  if (main_window)
+  {
+    gdk_window = gtk_widget_get_window (main_window);
+    if (gdk_window)
+      return gdk_screen_get_root_window (gdk_drawable_get_screen (gdk_window));
+  }
+  return NULL;
+}
+
 void
 gok_input_detach_corepointer (void)
 {
   int		num_devices, i;
+  GdkWindow    *root;
   Display      *display;
-  GtkWidget    *pMainWindow = gok_main_get_main_window ();
 #ifdef HAVE_XINPUT
   XDeviceInfo *devices;
-  if (!corepointer_detached && pMainWindow && pMainWindow->window) 
+
+  root = gok_input_get_root_window ();
+  if (!corepointer_detached && root)
   {
 	  XDeviceInfo *pointer = NULL, *ext = NULL;
-	  display = GDK_WINDOW_XDISPLAY (pMainWindow->window); 
+	  display = gok_main_display ();
 	  devices = XListInputDevices(display, &num_devices);
 	  for (i = 0; i < num_devices; i++) 
 	  {
@@ -179,12 +198,10 @@ gok_input_detach_corepointer (void)
 		  gint number;
 		  XEventClass ev_list[40];
 		  XDevice *newdev;
-		  GdkWindow       *root;
 	          saved_corepointer_id = pointer->id;
 		  XChangePointerDevice (display, XOpenDevice (display, ext->id), 0, 1);
 		  newdev = XOpenDevice (display, pointer->id);
 		  number = gok_input_init_device_event_list (pointer, newdev, ev_list);
-		  root = gdk_screen_get_root_window (gdk_drawable_get_screen (pMainWindow->window));
 		  if (XSelectExtensionEvent(GDK_WINDOW_XDISPLAY (root), 
 					    GDK_WINDOW_XWINDOW (root),
 					    ev_list, number)) 
@@ -212,11 +229,10 @@ void
 gok_input_restore_corepointer (void)
 {
   Display      *display;
-  GtkWidget    *pMainWindow = gok_main_get_main_window ();
 #ifdef HAVE_XINPUT
-  if (corepointer_detached && pMainWindow && pMainWindow->window) 
+  if (corepointer_detached)
   {
-	  display = GDK_WINDOW_XDISPLAY (pMainWindow->window); 
+	  display = gok_main_display ();
 	  XChangePointerDevice (display, XOpenDevice (display, saved_corepointer_id), 0, 1);
 	  corepointer_detached = FALSE;
   }
@@ -239,19 +255,14 @@ gok_input_ext_devices_exist (void)
 static gboolean   
 gok_input_open (GokInput *input)
 {
-  GtkWidget *window;
-
   g_return_val_if_fail (input != NULL, FALSE);
   input->open = FALSE;
 
 #ifdef HAVE_XINPUT
-  window = gok_main_get_main_window ();
-  if (window && window->window && 
-      input->info && (input->info->use == IsXExtensionDevice))
+  if (input->info && (input->info->use == IsXExtensionDevice))
     {
       gdk_error_trap_push ();
-      input->device = XOpenDevice(GDK_WINDOW_XDISPLAY (window->window), 
-				  input->info->id);
+      input->device = XOpenDevice(gok_main_display (), input->info->id);
       if (!gdk_error_trap_pop ())
 	  input->open = TRUE;
     }
@@ -265,14 +276,10 @@ gok_input_open (GokInput *input)
 static void
 gok_input_close (GokInput *input)
 {
-  GtkWidget *window = gok_main_get_main_window ();
-
-#ifdef HAVE_XINPUT  
-  if (window && window->window && 
-      input && input->info && (input->info->use == IsXExtensionDevice))
+#ifdef HAVE_XINPUT
+  if (input && input->info && (input->info->use == IsXExtensionDevice))
     {
-      XCloseDevice(GDK_WINDOW_XDISPLAY (window->window), 
-		   input->device);
+      XCloseDevice(gok_main_display (), input->device);
       input->open = FALSE;
     }
 #endif
@@ -285,13 +292,11 @@ gok_input_get_device_list (void)
   int		i;
   int		num_devices;
   Display      *display;
-  GtkWidget    *pMainWindow = gok_main_get_main_window ();
+#ifdef HAVE_XINPUT
   XDeviceInfo  *devices;
 
-#ifdef HAVE_XINPUT
-  if (!input_device_list && pMainWindow && pMainWindow->window) {
-	  display = GDK_WINDOW_XDISPLAY (pMainWindow->window); 
- 
+  if (!input_device_list) {
+	  display = gok_main_display ();
 	  devices = XListInputDevices(display, &num_devices);
 	  for (i = 0; i < num_devices; i++) {
 		  GokInput *pInput;
@@ -369,10 +374,13 @@ gok_input_init (GdkFilterFunc filter_func)
 #ifdef HAVE_XINPUT
 	XEventClass      event_list[40];
 	int              number;
-	GtkWidget       *window = gok_main_get_main_window ();
 	GdkWindow       *root;
 	GSList          *device_list = gok_input_get_device_list ();
-	
+
+	root = gok_input_get_root_window ();
+	if (!root)
+		return FALSE;
+
 	while (device_list && device_list->data) {
 	        GokInput *input = device_list->data;
 		if (!input->open && !gok_input_open (input)) {
@@ -380,8 +388,6 @@ gok_input_init (GdkFilterFunc filter_func)
 		}
 		else {
 		  number = gok_input_init_device_event_list (input->info, input->device, event_list);
-		  root = gdk_screen_get_root_window (gdk_drawable_get_screen (window->window));
-	
 		  if (XSelectExtensionEvent(GDK_WINDOW_XDISPLAY (root), 
 					    GDK_WINDOW_XWINDOW (root),
 					    event_list, number)) 
@@ -438,14 +444,9 @@ gok_input_free (GokInput *pInput)
 GokInput * 
 gok_input_find_by_device_id (guint id, gboolean extended_only)
 {
-  GtkWidget *window = gok_main_get_main_window ();
   GokInput *input = NULL;
 #ifdef HAVE_XINPUT
-  
-  if (window && window->window) 
-    {
-      input = find_input_by_id (GDK_WINDOW_XDISPLAY (window->window), id, extended_only);
-    }
+  input = find_input_by_id (gok_main_display (), id, extended_only);
 #else
   g_message ("Warning, XInput extension not present, cannot open input device.");
 #endif
@@ -455,13 +456,11 @@ gok_input_find_by_device_id (guint id, gboolean extended_only)
 GokInput * 
 gok_input_find_by_name (char *name, gboolean extended_only)
 {
-  GtkWidget *window = gok_main_get_main_window ();
   GokInput *input = NULL;
 #ifdef HAVE_XINPUT
-  
-  if (name && window && window->window) 
+  if (name)
     {
-      input = find_input (GDK_WINDOW_XDISPLAY (window->window), name, extended_only);
+      input = find_input (gok_main_display (), name, extended_only);
     }
 #else
   g_message ("Warning, XInput extension not present, cannot open input device.");
