@@ -4,7 +4,9 @@
 * Main file for the mighty GOK
 *
 * Copyright 2001-2009 Sun Microsystems, Inc.,
-* Copyright 2001-2009 University Of Toronto*
+* Copyright 2001-2009 University Of Toronto
+* Copyright 2009 Gerd Kohlberger <gerdko gmail com>
+*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
@@ -30,11 +32,11 @@
 #include <sys/types.h>
 #include <dirent.h>
 #include <signal.h>
-#include <libgnome/gnome-program.h>
-#include <libgnomeui/gnome-client.h>
-#include <libgnomeui/libgnomeui.h>
+#include <unistd.h>
+#include <errno.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
+#include <dbus/dbus-glib.h>
 #include <cspi/spi.h>
 #include <libbonobo.h>
 #include <bonobo-activation/bonobo-activation.h>
@@ -63,7 +65,6 @@
 #include "gok-sound.h"
 #include "gok-log.h"
 #include "gok-gconf-keys.h"
-#include "gok-predictor.h"
 #include "gok-editor.h"
 #include "gok-spy.h"
 #include "gok-feedback.h"
@@ -104,6 +105,13 @@ static Display *xevie_dpy = NULL;
 #define GCONF_ACCESSIBILITY_KEY "/desktop/gnome/interface/accessibility"
 #define KEYBOARD_ACCESSIBILITY_ENABLE_KEY "/desktop/gnome/accessibility/keyboard/enable"
 
+/* GNOME Session Manager */
+#define GSM_SERVICE_DBUS       "org.gnome.SessionManager"
+#define GSM_PATH_DBUS          "/org/gnome/SessionManager"
+#define GSM_INTERFACE_DBUS     "org.gnome.SessionManager"
+
+#define GSM_LOGOUT_MODE_NORMAL 0
+
 static gboolean respawn_on_segv = TRUE;
 
 static guint input_idle_handler = 0;
@@ -136,9 +144,6 @@ static gint m_GeometryHeight = -1;
 static gint m_GeometryX = -1;
 static gint m_GeometryY = -1;
 
-/* pointer to the gok command predictor */
-static Gok_Predictor m_pCommandPredictor = NULL;
-
 /* command line aguments storage structure */
 typedef struct _GokArgs {
 	char* accessmethodname;
@@ -174,7 +179,6 @@ static struct {
 static void gok_main_read_rc (void);
 static void gok_main_initialize_access_methods (GokArgs *args);
 static void gok_main_initialize_wordcomplete (void);
-static void gok_main_initialize_commandprediction (void);
 static gboolean gok_main_display_scan_main(void);
 static void gok_main_object_state_listener (Accessible* pAccessible);
 static gboolean gok_main_has_xkb_extension ();
@@ -433,7 +437,7 @@ gok_args_init (GokArgs *args)
 	args->custom_compose_kbd_name = NULL;
 	args->display_keyboard_editor = FALSE;
 	args->geometry = NULL;
-	args->geometry_bitmask;
+	args->geometry_bitmask = 0;
 	args->is_login = 0;
 	args->debug_nameless = FALSE;
 	args->remember_geometry = FALSE;
@@ -496,13 +500,18 @@ static Window*
 gok_application_get_raise_windows (LoginHelper *helper)
 {
     Window *mainwin = NULL;
+    GdkWindow *window;
     GtkWidget *widget = gok_main_get_main_window ();
 
     if (widget)
     {
-	mainwin = g_new0 (Window, 2);
-	mainwin[0] = GDK_WINDOW_XWINDOW (widget->window);
-	mainwin[1] = None;
+	window = gtk_widget_get_window (widget);
+	if (window)
+	{
+	    mainwin = g_new0 (Window, 2);
+	    mainwin[0] = GDK_WINDOW_XWINDOW (window);
+	    mainwin[1] = None;
+	}
     }
     return mainwin;
 }
@@ -615,13 +624,14 @@ void
 gok_main_center_corepointer (GtkWidget *window)
 {
     Display *display;
-    GdkWindow *root;
+    GdkWindow *root, *gdk_window;
     GdkScreen *screen;
     gint screen_width, screen_height;
 
     window = gok_main_get_main_window ();
-    display = GDK_WINDOW_XDISPLAY (window->window);
-    screen = gdk_drawable_get_screen (window->window);
+    gdk_window = gtk_widget_get_window (window);
+    display = GDK_WINDOW_XDISPLAY (gdk_window);
+    screen = gdk_drawable_get_screen (gdk_window);
     root = gdk_screen_get_root_window (screen);
     screen_width = gdk_screen_get_width (screen);
     screen_height = gdk_screen_get_height (screen);
@@ -643,9 +653,9 @@ gok_main_center_corepointer (GtkWidget *window)
 gint
 gok_main_open(gint argc, gchar *argv[])
 {
-	GnomeProgram *gok_program;
 	GOptionContext *context;
-	gboolean create_dirs;
+	GError *error = NULL;
+	gboolean create_dirs, parsed;
 	int i;
 
 	/* initialize member data */
@@ -658,23 +668,22 @@ gok_main_open(gint argc, gchar *argv[])
 
 	gok_args_init (&gok_args);
 
-	context	= g_option_context_new (_("The GNOME On-screen Keyboard"));
+	context = g_option_context_new (_("The GNOME On-screen Keyboard"));
 	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	parsed = g_option_context_parse (context, &argc, &argv, &error);
+	g_option_context_free (context);
+	if (!parsed)
+	{
+		g_print ("%s\n", error->message);
+		g_error_free (error);
+		_exit (-1);
+	}
 
 	/* If the user does not have a writable HOME directory, then
 	   avoid creating the directory. */
 	create_dirs = (g_access (g_get_home_dir(), W_OK) == 0);
 
-	/* Initialize Gnome */
-	gok_program = gnome_program_init (PACKAGE, VERSION,
-		LIBGNOMEUI_MODULE, argc, argv,
-		GNOME_PARAM_GOPTION_CONTEXT, context,
-		GNOME_PROGRAM_STANDARD_PROPERTIES,
-	        GNOME_PARAM_APP_DATADIR, DATADIR"/gok", 
-		LIBGNOMEUI_PARAM_CRASH_DIALOG, FALSE,
-		GNOME_PARAM_CREATE_DIRECTORIES, create_dirs,
-		NULL);
-		
 	if (!bonobo_init (&argc, argv))
 	{
 	    if (gok_args.is_login)
@@ -957,7 +966,6 @@ gok_main_open(gint argc, gchar *argv[])
 	gok_log ("login mode = %s\n", gok_args.is_login ? "true" : "false");
 	if (!gok_args.is_login) {
 		gok_main_initialize_wordcomplete ();
-		gok_main_initialize_commandprediction ();
 	}
 
 	/* create the settings dialog */
@@ -1012,8 +1020,6 @@ gok_log ("finished check");
 	signal(SIGTERM, (void (*)(int))gok_sig_handler);
 	signal(SIGINT,  (void (*)(int))gok_sig_handler);
 
-	g_object_unref (gok_program);
-
 	return 0;
 }
 
@@ -1089,6 +1095,7 @@ gok_main_set_cursor (GdkCursor *cursor)
 {
 	GdkColor color = { 0, 0, 0, 0 };
 	static GdkCursor *nilCursor = NULL;
+	GtkWidget *main;
 	if (!cursor) {
 		if (!nilCursor) {
 			GdkPixmap *source = 
@@ -1099,19 +1106,8 @@ gok_main_set_cursor (GdkCursor *cursor)
 		}		
 		cursor = nilCursor;
 	}
-	gdk_window_set_cursor (gok_main_get_main_window ()->window, cursor);
-}
-
-/**
-* gok_main_get_command_predictor
-* 
-* Accessor function.
-*
-* returns: A pointer to command prediction engine
-*/
-Gok_Predictor gok_main_get_command_predictor ()
-{
-	return m_pCommandPredictor;
+	main = gok_main_get_main_window ();
+	gdk_window_set_cursor (gtk_widget_get_window (main), cursor);
 }
 
 /**
@@ -1368,6 +1364,7 @@ gok_main_queue_input_event (int *motion_data, gint n_axes)
 */
 void gok_main_motion_listener (gint n_axes, int *motion_data, long mods, long timestamp)
 {
+	GdkWindow *gdk_window;
         GtkWidget *window = gok_main_get_main_window ();
 	if (window) 
 	{
@@ -1380,12 +1377,12 @@ void gok_main_motion_listener (gint n_axes, int *motion_data, long mods, long ti
 #endif
 
 		gok_main_queue_input_event (motion_data, n_axes);
-
-		if (gok_data_get_drive_corepointer ()) {
+		gdk_window = gtk_widget_get_window (window);
+		if (gok_data_get_drive_corepointer () && gdk_window) {
 			Display *display;
 			GdkWindow *root;
-			display = GDK_WINDOW_XDISPLAY (window->window);
-			root = gdk_screen_get_root_window (gdk_drawable_get_screen (window->window));
+			display = GDK_WINDOW_XDISPLAY (gdk_window);
+			root = gdk_screen_get_root_window (gdk_drawable_get_screen (gdk_window));
 			XWarpPointer (display, None, GDK_WINDOW_XWINDOW (root), 0, 0, 0, 0, motion_data[0], motion_data[1]);
 		}
 	}
@@ -1411,11 +1408,14 @@ void gok_main_button_listener (gint button, gint state, long mods, long timestam
 		Window root_ret, child_ret;
 		unsigned int mask_ret;
 		GtkWidget *window;
+		GdkWindow *gdk_window;
 		char ename[12];
+
 		snprintf (ename, 12, "b%d%c", button, (state) ? 'p' : 'r');
 		window = gok_main_get_main_window ();
-		XQueryPointer (GDK_WINDOW_XDISPLAY (window->window), 
-			       GDK_WINDOW_XWINDOW (window->window), 
+		gdk_window = gtk_widget_get_window (window);
+		XQueryPointer (GDK_WINDOW_XDISPLAY (gdk_window),
+			       GDK_WINDOW_XWINDOW (gdk_window),
 			       &root_ret, &child_ret, &x, &y,
 			       &win_x, &win_y, &mask_ret);
 		SPI_generateMouseEvent (x, y, ename);
@@ -1486,9 +1486,11 @@ void gok_main_button_listener (gint button, gint state, long mods, long timestam
 void
 gok_main_raise_window (void)
 {
-	if (m_pWindowMain && m_pWindowMain->window) 
+	GdkWindow *window;
+
+	if (m_pWindowMain && (window = gtk_widget_get_window (m_pWindowMain)))
 	{
-		gdk_window_raise (m_pWindowMain->window);
+		gdk_window_raise (window);
 	}
 }
 
@@ -1878,8 +1880,6 @@ gboolean gok_main_ds (GokKeyboard* pKeyboard)
 	/* enable/disable the menu and toolbar keys */
 	gok_keyboard_validate_dynamic_keys (gok_main_get_foreground_window_accessible());
 
-	/* if there are command prediction keys - fill them in */
-	gok_predictor_predict(m_pCommandPredictor);
 
 	/* initialize the current access method */
 	gok_scanner_reset_access_method ();
@@ -2391,49 +2391,6 @@ gok_main_initialize_wordcomplete ()
     g_free (directory_name);
 }
 
-/**
-* gok_main_initialize_commandpredictione:
-*
-* Retreives the name of the directory containing dictionary.txt from
-* GConf and calls gok_wordcomplete_open passing that directory.
-*/
-static void
-gok_main_initialize_commandprediction ()
-{
-	gok_log_enter();
-    /* intialize command predictor */
-    m_pCommandPredictor = gok_predictor_open();
-   /*
-    GConfClient *gconf_client = NULL;
-    GError *gconf_err = NULL;
-    gchar *directory_name = NULL;
-
-    gconf_client = gconf_client_get_default ();
-    
-    directory_name = gconf_client_get_string (gconf_client,
-					      GOK_GCONF_DICTIONARY_DIRECTORY,
-					      &gconf_err);
-    if (directory_name == NULL)
-    {
-	gok_log_x ("Got NULL dictionary directory key from GConf");
-    }
-    else if (gconf_err != NULL)
-    {
-	gok_log_x ("Error getting dictionary directory key from GConf");
-	g_error_free (gconf_err);
-    }
-    else
-    {
-	if ( !(gok_wordcomplete_open (directory_name)) ) {
-	    gok_log_x ("Error initializing word completion");
-	}
-	
-	g_free (directory_name);
-    }
-    */
-	gok_log_leave();
-}
-
 /**
 * gok_main_resize_window
 * @pWindow: Pointer to the main window.
@@ -2460,6 +2417,7 @@ void gok_main_resize_window (GtkWidget* pWindow, GokKeyboard *pKeyboard, gint Wi
 	gint screenX;
 	gint screenY;
 	GdkRectangle rectFrame;
+	GdkWindow *gdk_window;
 
 	/* ensure the window is at least this big */
 	if (Width < 50)
@@ -2509,9 +2467,10 @@ void gok_main_resize_window (GtkWidget* pWindow, GokKeyboard *pKeyboard, gint Wi
 
 	/* add the frame dimension to the window dimension */
 	/* get the frame location */
-	gdk_window_get_frame_extents ((GdkWindow*)pWindow->window, &rectFrame);
+	gdk_window = gtk_widget_get_window (pWindow);
+	gdk_window_get_frame_extents (gdk_window, &rectFrame);
 	/* get the window location */
-	gdk_window_get_position (pWindow->window, &winX, &winY);
+	gdk_window_get_position (gdk_window, &winX, &winY);
 
 	if ((winX != 0) &&
 		(winY != 0))
@@ -2569,7 +2528,7 @@ void gok_main_resize_window (GtkWidget* pWindow, GokKeyboard *pKeyboard, gint Wi
 	}
 
 	/* move/resize the main window */
-	gdk_window_move_resize ((GdkWindow*)pWindow->window, left, top, Width, Height);
+	gdk_window_move_resize (gdk_window, left, top, Width, Height);
 
 	if (gok_data_get_dock_type () != GOK_DOCK_NONE) {
 		/* if this is a "never expand" keyboard, unset struts! */
@@ -2589,8 +2548,10 @@ gok_main_set_wm_dock (gboolean is_dock)
 {
   Atom atom_type[1], atom_window_type;
   GtkWidget *widget = gok_main_get_main_window ();
+  GdkWindow *window;
 
   if (widget) gtk_widget_hide (widget);
+  window = gtk_widget_get_window (widget);
   gdk_error_trap_push ();
   atom_window_type = gdk_x11_get_xatom_by_name ("_NET_WM_WINDOW_TYPE");
 
@@ -2599,8 +2560,8 @@ gok_main_set_wm_dock (gboolean is_dock)
   else 
 	  atom_type[0] = gdk_x11_get_xatom_by_name ("_NET_WM_WINDOW_TYPE_NORMAL");
 
-  XChangeProperty (GDK_WINDOW_XDISPLAY (widget->window), 
-		   GDK_WINDOW_XWINDOW (widget->window), 
+  XChangeProperty (GDK_WINDOW_XDISPLAY (window),
+		   GDK_WINDOW_XWINDOW (window),
 		   atom_window_type,
 		   XA_ATOM, 32, PropModeReplace,
 		   (guchar *) &atom_type, 1);
@@ -2640,6 +2601,7 @@ void
 gok_main_update_struts (gint width, gint height, gint x, gint y)
 {
 	GtkWidget *widget = gok_main_get_main_window ();
+	GdkWindow *window;
 	Atom atom_strut = gdk_x11_get_xatom_by_name ("_NET_WM_STRUT");
 	Atom atom_strut_partial = gdk_x11_get_xatom_by_name ("_NET_WM_STRUT_PARTIAL");
 	guint32 struts[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
@@ -2661,15 +2623,15 @@ gok_main_update_struts (gint width, gint height, gint x, gint y)
 	}
 	gdk_error_trap_push ();
 	/* this means that we are responsible for placing ourselves appropriately on screen */
-	if (widget && widget->window)
+	if (widget && (window = gtk_widget_get_window (widget)))
 	{
-		XChangeProperty (GDK_WINDOW_XDISPLAY (widget->window), 
-				 GDK_WINDOW_XWINDOW (widget->window), 
+		XChangeProperty (GDK_WINDOW_XDISPLAY (window),
+				 GDK_WINDOW_XWINDOW (window),
 				 atom_strut,
 				 XA_CARDINAL, 32, PropModeReplace,
 				 (guchar *) &struts, 4);
-		XChangeProperty (GDK_WINDOW_XDISPLAY (widget->window), 
-				 GDK_WINDOW_XWINDOW (widget->window), 
+		XChangeProperty (GDK_WINDOW_XDISPLAY (window),
+				 GDK_WINDOW_XWINDOW (window),
 				 atom_strut_partial,
 				 XA_CARDINAL, 32, PropModeReplace,
 				 (guchar *) &struts, 12);
@@ -2701,6 +2663,7 @@ void gok_main_store_window_center ()
 	gint winY;
 	gint winWidthCurrent;
 	gint winHeightCurrent;
+	GdkWindow *window;
 
 	/* don't store the window center if geometry is specified */
 	if (m_bUseGivenGeometry == TRUE)
@@ -2716,8 +2679,9 @@ void gok_main_store_window_center ()
 	}
 
 	/* get the center of the current window */	
-	gdk_window_get_position (m_pWindowMain->window, &winX, &winY);
-	gdk_drawable_get_size (m_pWindowMain->window, &winWidthCurrent, &winHeightCurrent);
+	window = gtk_widget_get_window (m_pWindowMain);
+	gdk_window_get_position (window, &winX, &winY);
+	gdk_drawable_get_size (window, &winWidthCurrent, &winHeightCurrent);
 	winX +=  winWidthCurrent / 2;	
 	winY +=  winHeightCurrent / 2;
 
@@ -2800,7 +2764,6 @@ gok_main_close()
 	gok_feedback_close();
 	gok_control_button_callback_close();
 	gok_wordcomplete_close(gok_wordcomplete_get_default ());
-	gok_predictor_close(m_pCommandPredictor);
 	
 	_exit(0);
 }
@@ -2929,12 +2892,15 @@ gok_main_window_contains_pointer (void)
 	gint x=-1, y=-1, w=0, h=0;
 	gint px, py;
 	GtkWidget *widget = gok_main_get_main_window ();
+	GdkWindow *window;
+
 	gok_scanner_get_pointer_location (&px, &py);
-	if (widget->window) {
-	  gdk_window_get_position (widget->window, &x, &y);
+	window = gtk_widget_get_window (widget);
+	if (window) {
+	  gdk_window_get_position (window, &x, &y);
 	  x = px-x;
 	  y = py-y;
-	  gdk_drawable_get_size (widget->window, &w, &h);
+	  gdk_drawable_get_size (window, &w, &h);
 	}
 	if (x < 0 || y < 0 || x > w || y > h)
 	  return FALSE;
@@ -2945,8 +2911,10 @@ gok_main_window_contains_pointer (void)
 Display *
 gok_main_display (void)
 {
-	if (gok_main_get_main_window () != NULL)
-		return GDK_WINDOW_XDISPLAY (gok_main_get_main_window ()->window);
+	GtkWidget *widget = gok_main_get_main_window ();
+	GdkWindow *window;
+	if (widget && (window = gtk_widget_get_window (widget)))
+		return GDK_WINDOW_XDISPLAY (window);
 	return GDK_DISPLAY ();
 }
 
@@ -3089,11 +3057,10 @@ static gboolean
 gok_main_check_sticky_keys (GtkWidget *widget)
 {
 	int op_rtn, event_rtn, error_rtn;
-	Display *display = GDK_WINDOW_XDISPLAY (widget->window);
+	Display *display = gok_main_display ();
 	GtkWidget *dialog;
 	gboolean xkb_ok = FALSE;
 
-	g_assert (display);
 	if (XkbQueryExtension (display,
 			       &op_rtn, &event_rtn, &error_rtn, NULL, NULL)) {
 		XkbDescRec *xkb;
@@ -3168,24 +3135,72 @@ static GtkWidget* _corepointer_warning = NULL; /* core pointer warning dialog */
 
 static gboolean _corepointer_warning_posted = FALSE; /* core pointer warning dialog has been posted */
 
-void check_accessibility_cb ( GObject* o, gpointer* data )
+/**
+ * gok_main_request_logout:
+ *
+ * Calls the 'Logout' method of gnome-session-manager over DBus.
+ * The 'mode' argument is set to 0 (normal).
+ * Other modes are 1 (no confirmation) and 2 (force).
+ *
+ * Returns: #TRUE on success, #FALSE otherwise.
+ */
+static gboolean
+gok_main_request_logout (void)
 {
-	GnomeClient *client;
+	DBusGConnection *bus;
+	DBusGProxy *sm_proxy;
+	GError *error = NULL;
+	gboolean ret = FALSE;
 
-	if (acd) { 
-	    gtk_widget_destroy(acd); acd = NULL; 
-	}	
-	
+	gok_log_enter ();
+
+	/* get session bus */
+	bus = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+	if (bus)
+	{
+		/* create g-s-m proxy */
+		sm_proxy = dbus_g_proxy_new_for_name (bus,
+						      GSM_SERVICE_DBUS,
+						      GSM_PATH_DBUS,
+						      GSM_INTERFACE_DBUS);
+		/* request logout (sync. call) */
+		ret = dbus_g_proxy_call (sm_proxy, "Logout", &error,
+					 G_TYPE_UINT, GSM_LOGOUT_MODE_NORMAL,
+					 G_TYPE_INVALID, G_TYPE_INVALID);
+		g_object_unref (sm_proxy);
+		if (error)
+		{
+			gok_log ("Logout method: %s", error->message);
+			g_error_free (error);
+		}
+	}
+	else
+	{
+		gok_log ("Couldn't connect to session bus.");
+	}
+	gok_log_leave ();
+
+	return ret;
+}
+
+static void
+check_accessibility_cb (GtkButton *button, gpointer data)
+{
+	if (acd) {
+		gtk_widget_destroy(acd);
+		acd = NULL;
+	}
+
 	if (strcmp((gchar*)data,"logout") == 0) {
-		gok_gconf_set_bool ( gok_data_get_gconf_client(),
-			GCONF_ACCESSIBILITY_KEY, TRUE );
-		if (!(client = gnome_master_client())) {
-			gok_main_close();
+		/* enable gnome a11y key */
+		gok_gconf_set_bool (gok_data_get_gconf_client (),
+				    GCONF_ACCESSIBILITY_KEY, TRUE);
+
+		/* request logout */
+		if (!gok_main_request_logout ()) {
+			/* exit if logout fails */
+			gok_main_close ();
 		}
- 		gnome_client_request_save (client, GNOME_SAVE_GLOBAL, TRUE,
-			GNOME_INTERACT_ANY, FALSE, TRUE); /* code borrowed from 
-			gnome-control-center/ capplets/ accessibility/ at-properties/ 
-			main.c */
 	}
 	else if (strcmp((gchar*)data,"quit") == 0) {
 		gok_main_close();
@@ -3194,7 +3209,6 @@ void check_accessibility_cb ( GObject* o, gpointer* data )
 	else if (strcmp((gchar*)data,"continue") == 0) {
 		/* maybe change gok somehow to show user weakened status */
 	}
-
 }
 
 /**
@@ -3297,7 +3311,7 @@ gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc, GCallback
 
 	if (!gok_main_get_use_corepointer() && ( always || gok_scanner_current_state_uses_corepointer() )) 
 	{
-		GtkWidget*	button;
+		GtkWidget *button, *action_area;
 
 		if (opt_out && okfunc) (*okfunc) ();
 
@@ -3309,15 +3323,15 @@ gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc, GCallback
 				GTK_MESSAGE_WARNING, 
 				GTK_BUTTONS_NONE,			
 				"%s", message);
-			
+			action_area = gtk_dialog_get_action_area (GTK_DIALOG(_corepointer_warning));
 			button = gtk_button_new_from_stock(GTK_STOCK_HELP);
-			gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
+			gtk_container_add (GTK_CONTAINER (action_area),
 					   button);
 			g_signal_connect (button, "clicked",
 					  G_CALLBACK (gok_main_help_cb), NULL);
 			
 			button = gtk_button_new_from_stock(GTK_STOCK_PREFERENCES);
-			gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
+			gtk_container_add (GTK_CONTAINER (action_area),
 					   button);
 			g_signal_connect (G_OBJECT (button), "clicked", 
 					  G_CALLBACK (gok_settingsdialog_show), NULL);
@@ -3325,13 +3339,13 @@ gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc, GCallback
 			if (cancelfunc != NULL)
 			{
 			    button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-			    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
+			    gtk_container_add (GTK_CONTAINER (action_area),
 					       button);
 			    g_signal_connect_swapped (G_OBJECT (button), "clicked", 
 						      cancelfunc, _corepointer_warning);
 			}
 			button = gtk_button_new_from_stock(GTK_STOCK_OK);
-			gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
+			gtk_container_add (GTK_CONTAINER (action_area),
 					   button);
 			g_signal_connect_swapped (G_OBJECT (button), "clicked", 
 						  (okfunc == NULL || opt_out) ? G_CALLBACK (gok_main_close_warning) : okfunc, 
@@ -3359,7 +3373,7 @@ gok_main_check_accessibility ()
 	gok_gconf_get_bool ( client, GCONF_ACCESSIBILITY_KEY, &accessibility_on );
         
     if (!accessibility_on) {
-		GtkWidget*	button;
+		GtkWidget *button, *action_area;
 		
 		acd = gtk_message_dialog_new (
 			NULL, 
@@ -3385,20 +3399,21 @@ gok_main_check_accessibility ()
 		"Close" "." 
 		"\n\n"));
 
+		action_area = gtk_dialog_get_action_area (GTK_DIALOG(acd));
 		button = gtk_button_new_with_label(_("Enable and Log Out"));
-		gtk_container_add (GTK_CONTAINER (GTK_DIALOG(acd)->action_area),
+		gtk_container_add (GTK_CONTAINER (action_area),
 						  button);
 		g_signal_connect(G_OBJECT (button), "clicked", 
 			G_CALLBACK (check_accessibility_cb), (gpointer) "logout");
 			
 		button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
-		gtk_container_add (GTK_CONTAINER (GTK_DIALOG(acd)->action_area),
+		gtk_container_add (GTK_CONTAINER (action_area),
 						  button);
 		g_signal_connect(G_OBJECT (button), "clicked", 
 			G_CALLBACK (check_accessibility_cb), (gpointer) "quit");
 
 		button = gtk_button_new_with_label(_("Continue"));
-		gtk_container_add (GTK_CONTAINER (GTK_DIALOG(acd)->action_area),
+		gtk_container_add (GTK_CONTAINER (action_area),
 						  button);
 		g_signal_connect(G_OBJECT (button), "clicked", 
 			G_CALLBACK (check_accessibility_cb), (gpointer) "continue");
